# Copyright (c) 2004-2013  Pavel Rychly, Vojtech Kovar, Milos Jakubicek, Vit Baisa

import CGIPublisher
import os
 
def load_opt_file (options, filepath):
    if not os.path.isfile (filepath):
        return
    for line in open (filepath).readlines():
        if line[0] == '#':
            continue
        a,v = line.split ('\t', 1)
        options [a.strip()] = v.strip('\n').decode('utf8')


class UserCGI (CGIPublisher.CGIPublisher):
    _ca_user_info = u''
    _ca_languages = []
    _options_dir = u''
    _email = u''
    _default_user = u'defaults'
    attrs2save = []

    def __init__ (self, user=None):
        CGIPublisher.CGIPublisher.__init__ (self)
        self._user = user

    def _user_defaults (self, user):
        pass

    def _get_ca_corpnames(self, language='', with_ws=0):
        import urllib2, simplejson
        request = urllib2.Request(self._ca_corp_info
                % (self._sessionid, self._user_ip, language, with_ws))
        file = urllib2.urlopen(request)
        data = file.read()
        return simplejson.loads(data)

    def _get_ca_languages(self):
        import urllib2, simplejson
        request = urllib2.Request(self._ca_lang_info)
        file = urllib2.urlopen(request)
        data = file.read()
        self._ca_languages = simplejson.loads(data)

    def _get_ca_user_info(self, corpname=''):
        import urllib, urllib2, simplejson
        if self._login_address:
            self._login_address += '/?next=' + urllib.quote_plus(
                                                os.getenv ('REQUEST_URI', '-'))
        request = urllib2.Request(self._ca_user_info
                                  % (self._sessionid, self._user_ip, corpname))
        no_tries = 0
        while True:
            try:
                file = urllib2.urlopen(request)
                break
            except urllib2.HTTPError:
                import time
                if no_tries <= 5: time.sleep(3); no_tries += 1
                else: raise
        data = file.read()
        file.close()
        user_info = simplejson.loads(data)
        if user_info.has_key('error'): self._has_access = False
            # set attributes (incl. these starting '_') and correct types
        corr_func = {type(0): int, type(0.0): float, type([]): lambda x: [x]}
        for k, v in user_info.items():
            try: setattr (self, k, corr_func[type(getattr(self, k))](v))
            except: setattr (self, k, v)
        if self._user: self._user = str(self._user)

    def _setup_user (self, corpname=''):
        if self._ca_user_info:
            self._has_access = 0
            self._get_ca_user_info(corpname)
        if hasattr(self, '_ca_lang_info'):
            self._get_ca_languages()
        if not self._user:
            self._user = os.getenv ('REMOTE_USER','-')
            if self._user == '-':
                self._user = self._default_user
        user = self._user
        options = {}
        load_opt_file (options, os.path.join (self._options_dir,
                                              self._default_user))
        if user is not self._default_user:
            load_opt_file (options, os.path.join (self._options_dir, user))
        CGIPublisher.correct_types (options, self.clone_self(), selector=1)
        self._user_defaults (user)
        self.__dict__.update (options)

    def _save_options (self, optlist=[], selector=''):
        if selector:
            tosave = [(selector + ':' + opt, self.__dict__[opt])
                           for opt in optlist if self.__dict__.has_key (opt)]
        else:
            tosave = [(opt, self.__dict__[opt]) for opt in optlist
                           if self.__dict__.has_key (opt)]
        opt_filepath = os.path.join (self._options_dir, self._user)
        options = {}
        load_opt_file(options, opt_filepath)
        options.update(tosave)

        opt_lines = []
        for k, v in options.iteritems():
            if isinstance(v, unicode): v = v.encode('utf8')
            opt_lines.append(str(k) + '\t' + str(v))
        opt_lines.sort()
        opt_file = open(opt_filepath, 'w')
        opt_file.write('\n'.join(opt_lines))
        opt_file.close()

    def save_global_attrs(self):
        options = [a for a in self.attrs2save if not a.startswith('_')]
        self._save_options(options, '')
        return _('Selected options successfully saved')

    def send_mail(self, subject, msg, sender='', recipient='', attachment=''):
        import smtplib
        import urllib
        if not recipient:
            recipient = self._email
        if not sender:
            sender = 'support@sketchengine.co.uk'
        server = smtplib.SMTP('localhost')
        mhead = ("From: <%s>" + \
                "\nTo: %s" + \
                "\nSubject: %s\n\n") % (sender, recipient, subject)
        msg = urllib.unquote(msg).encode('utf-8')
        if attachment[0]:
            from email.MIMEMultipart import MIMEMultipart
            from email.MIMEBase import MIMEBase
            from email.MIMEText import MIMEText
            from email.Utils import formatdate
            from email import Encoders
            m = MIMEMultipart()
            m['From'] = sender
            m['To'] = recipient
            m['Date'] = formatdate(localtime=True)
            m['Subject'] = subject
            m.attach(MIMEText(msg))
            part = MIMEBase('application', "octet-stream")
            part.set_payload(attachment[1].read())
            Encoders.encode_base64(part)
            part.add_header('Content-Disposition', 'attachment; filename="%s"' % attachment[0])
            m.attach(part)
            server.sendmail(sender, recipient, m.as_string())
        else:
            server.sendmail(sender, recipient, mhead.encode('utf-8') + msg)
        server.quit()

    def feedback(self, feedback_url='', navigator='', feedback_corpname='',
            feedback_text='', attachment=('', None)):
        msg = """
Feedback from SkE
=================

%s

USER: %s
CORPUS: %s
URL: %s
BROWSER: %s
""" % (feedback_text, self._user, feedback_corpname, feedback_url, navigator)
        self.send_mail('Feedback from SkE', msg, self._email or 'anonymous@anonym.com',
                'support@sketchengine.co.uk', attachment)
        return _('Your feedback has been sent to support@sketchengine.co.uk.' + \
                ' Thank you for your time!')
